use actix_web::{web, Responder};
use askama::Template;
use chrono::{Datelike, NaiveDate, Utc};
use sqlx::PgPool;

use crate::{
    models::{Area, Role, User},
    utils::{ApplicationError, TemplateResponse},
};

#[derive(Template)]
#[template(path = "export/events.html")]
struct EventExportTemplate {
    user: User,
    areas: Option<Vec<Area>>,
    daterange: (NaiveDate, NaiveDate),
}

#[actix_web::get("/export/events")]
pub async fn get(
    user: web::ReqData<User>,
    pool: web::Data<PgPool>,
) -> Result<impl Responder, ApplicationError> {
    if user.role != Role::Admin && user.role != Role::AreaManager {
        return Err(ApplicationError::Unauthorized);
    }

    let areas = if user.role == Role::Admin {
        Some(Area::read_all(pool.get_ref()).await?)
    } else {
        None
    };

    let today = Utc::now().date_naive();
    let start = NaiveDate::from_ymd_opt(today.year(), today.month0() + 1, 1).unwrap();
    let end = NaiveDate::from_ymd_opt(today.year(), today.month0() + 2, 1)
        .unwrap()
        .pred_opt()
        .unwrap();

    let template = EventExportTemplate {
        user: user.into_inner(),
        areas,
        daterange: (start, end),
    };

    Ok(template.to_response()?)
}
